@@ -1,6 +1,7 @@
 // This file is part of retro-gtk. License: GPL-3.0+.
 
 #include <retro-gtk/retro-gtk.h>
+#include <retro-gtk/retro-core-view.h>
 
 #define RETRO_TYPE_DEMO_APPLICATION (retro_demo_application_get_type())
 
@@ -13,10 +14,8 @@ struct _RetroDemoApplication
   GtkApplicationWindow *window;
 
   RetroCore *core;
-  RetroInputDeviceManager *input;
   RetroMainLoop *loop;
   RetroCoreView *view;
-  RetroPaPlayer *pa_player;
 };
 
 G_DEFINE_TYPE (RetroDemoApplication, retro_demo_application, GTK_TYPE_APPLICATION)
@@ -46,7 +45,7 @@ retro_demo_open (GApplication  *application,
   if (self->core == NULL)
     return;
 
-  retro_core_init (self->core, &error);
+  retro_core_boot (self->core, &error);
   if (error != NULL) {
     g_debug ("Couldn't initialize the Libretro core: %s", error->message);
     g_error_free (error);
@@ -54,9 +53,6 @@ retro_demo_open (GApplication  *application,
     return;
   }
 
-  self->input = retro_input_device_manager_new ();
-  retro_core_set_input_interface (self->core, RETRO_INPUT (self->input));
-
   g_application_activate (application);
 }
 
@@ -74,15 +70,9 @@ retro_demo_application_finalize (GObject *object)
   if (self->core != NULL)
     g_object_unref (self->core);
 
-  if (self->input != NULL)
-    g_object_unref (self->input);
-
   if (self->loop != NULL)
     g_object_unref (self->loop);
 
-  if (self->pa_player != NULL)
-    g_object_unref (self->pa_player);
-
   G_OBJECT_CLASS (retro_demo_application_parent_class)->finalize (object);
 }
 
@@ -92,7 +82,7 @@ retro_demo_activate (GApplication *application)
 {
   RetroDemoApplication *self;
   GtkWidget *window;
-  RetroInputDevice *input_device;
+  RetroController *controller;
 
   self = RETRO_DEMO_APPLICATION (application);
 
@@ -101,20 +91,19 @@ retro_demo_activate (GApplication *application)
   self->view = retro_core_view_new ();
   retro_core_view_set_core (self->view, self->core);
 
-  retro_input_device_manager_set_keyboard (self->input, GTK_WIDGET (self->view));
-  input_device = retro_core_view_as_input_device (self->view, RETRO_DEVICE_TYPE_POINTER);
-  retro_input_device_manager_set_controller_device (self->input, 0, input_device);
-  g_object_unref (input_device);
-
-  self->pa_player = retro_pa_player_new ();
-  retro_pa_player_set_core (self->pa_player, self->core);
+  retro_core_set_keyboard (self->core, GTK_WIDGET (self->view));
+  controller = retro_core_view_as_controller (self->view,
+                                              RETRO_CONTROLLER_TYPE_POINTER);
+  retro_core_set_controller (self->core, 0, controller);
+  g_object_unref (controller);
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_default_size (GTK_WINDOW (window), 640, 480);
   gtk_container_add (GTK_CONTAINER (window), GTK_WIDGET (self->view));
 
   gtk_widget_show_all (GTK_WIDGET (window));
-  gtk_application_add_window (GTK_APPLICATION (application), GTK_WINDOW (window));
+  gtk_application_add_window (GTK_APPLICATION (application),
+                              GTK_WINDOW (window));
 
   self->loop = retro_main_loop_new (self->core);
   retro_main_loop_start (self->loop);
