@@ -1,8 +1,9 @@
 // This file is part of retro-gtk. License: GPL-3.0+.
 
-#include "retro-gtk-internal.h"
+#include "retro-core-private.h"
+
 #include "libretro-environment.h"
-#include "retro-core.h"
+#include "retro-pixdata-private.h"
 
 void retro_core_set_system_av_info (RetroCore         *self,
                                     RetroSystemAvInfo *system_av_info);
@@ -32,34 +33,33 @@ rumble_callback_set_rumble_state (guint             port,
                                   RetroRumbleEffect effect,
                                   guint16           strength)
 {
-  RetroCore *cb_data;
-  RetroRumble *interface;
+  RetroCore *self;
+  RetroController *controller;
 
-  cb_data = retro_core_get_cb_data ();
-  if (!cb_data)
-    g_return_val_if_reached (FALSE);
+  self = retro_core_get_cb_data ();
 
-  interface = retro_core_get_rumble_interface (cb_data);
-  if (!interface)
-    g_return_val_if_reached (FALSE);
+  g_return_val_if_fail (RETRO_IS_CORE (self), FALSE);
 
-  return RETRO_RUMBLE_GET_INTERFACE (interface)->set_rumble_state (interface,
-                                                                   port,
-                                                                   effect,
-                                                                   strength);
+  if (!g_hash_table_contains (self->controllers, &port))
+    return FALSE;
+
+  controller = g_hash_table_lookup (self->controllers, &port);
+
+  if (controller == NULL)
+    return FALSE;
+
+  return retro_controller_set_rumble_state (controller, effect, strength);
 }
 
 static void
 on_log (guint level, const gchar *format, ...)
 {
   RetroCore *self;
-  RetroSystemInfo info = { 0 };
   const gchar *log_domain;
   GLogLevelFlags log_level;
   gchar *message;
   va_list args;
 
-
   self = retro_core_get_cb_data ();
   if (!self)
     g_return_if_reached ();
@@ -122,14 +122,7 @@ static gboolean
 get_input_device_capabilities (RetroCore *self,
                                guint64   *capabilities)
 {
-  RetroInput *input;
-
-  input = retro_core_get_input_interface (self);
-
-  if (input == NULL)
-    return FALSE;
-
-  *capabilities = retro_input_get_device_capabilities (input);
+  *capabilities = retro_core_get_controller_capabilities (self);
 
   return TRUE;
 }
@@ -156,7 +149,7 @@ static gboolean
 get_overscan (RetroCore *self,
               gboolean  *overscan)
 {
-  *overscan = RETRO_CORE_ENVIRONMENT_INTERNAL (self)->overscan;
+  *overscan = self->overscan;
 
   return TRUE;
 }
@@ -165,10 +158,6 @@ static gboolean
 get_rumble_callback (RetroCore           *self,
                      RetroRumbleCallback *cb)
 {
-  gpointer interface_exists = retro_core_get_rumble_interface (self);
-  if (!interface_exists)
-    return FALSE;
-
   cb->set_rumble_state = rumble_callback_set_rumble_state;
 
   return TRUE;
@@ -196,20 +185,22 @@ static gboolean
 get_variable (RetroCore     *self,
               RetroVariable *variable)
 {
-  gchar *result;
+  const gchar *value;
 
-  result = retro_variables_get_variable (self->variables_interface,
-                                         variable->key);
-  variable->value = result ? result : "";
+  if (!retro_options_contains (self->options, variable->key))
+    return FALSE;
 
-  return !!result;
+  value = retro_options_get_option_value (self->options, variable->key);
+  variable->value = g_strdup (value); // FIXME Is that a memory leak?
+
+  return TRUE;
 }
 
 static gboolean
 get_variable_update (RetroCore *self,
                      gboolean  *update)
 {
-  *update = retro_variables_get_variable_update (self->variables_interface);
+  *update = retro_options_get_variable_update (self->options);
 
   return TRUE;
 }
@@ -218,7 +209,7 @@ static gboolean
 set_disk_control_interface (RetroCore                *self,
                             RetroDiskControlCallback *callback)
 {
-  RETRO_CORE_ENVIRONMENT_INTERNAL (self)->disk_control_callback = callback;
+  self->disk_control_callback = callback;
 
   return TRUE;
 }
@@ -227,16 +218,10 @@ static gboolean
 set_input_descriptors (RetroCore            *self,
                        RetroInputDescriptor *descriptors)
 {
-  RetroInput *input;
   int length;
 
-  input = retro_core_get_input_interface (self);
-
-  if (input == NULL)
-    return FALSE;
-
   for (length = 0 ; descriptors[length].description ; length++);
-  retro_input_set_descriptors (input, descriptors, length);
+  retro_core_set_controller_descriptors (self, descriptors, length);
 
   return TRUE;
 }
@@ -245,7 +230,7 @@ static gboolean
 set_keyboard_callback (RetroCore             *self,
                        RetroKeyboardCallback *callback)
 {
-  RETRO_CORE_ENVIRONMENT_INTERNAL (self)->keyboard_callback = *callback;
+  self->keyboard_callback = *callback;
 
   return TRUE;
 }
@@ -264,7 +249,7 @@ static gboolean
 set_pixel_format (RetroCore              *self,
                   const RetroPixelFormat *pixel_format)
 {
-  RETRO_CORE_ENVIRONMENT_INTERNAL (self)->pixel_format = *pixel_format;
+  self->pixel_format = *pixel_format;
 
   return TRUE;
 }
@@ -273,7 +258,7 @@ static gboolean
 set_rotation (RetroCore           *self,
               const RetroRotation *rotation)
 {
-  RETRO_CORE_ENVIRONMENT_INTERNAL (self)->rotation = *rotation;
+  self->rotation = *rotation;
 
   return TRUE;
 }
@@ -300,13 +285,10 @@ static gboolean
 set_variables (RetroCore     *self,
                RetroVariable *variable_array)
 {
-  int length;
+  int i;
 
-  for (length = 0 ;
-       variable_array[length].key && variable_array[length].value ;
-       length++);
-  retro_variables_set_variable (self->variables_interface,
-                                variable_array, length);
+  for (i = 0 ; variable_array[i].key && variable_array[i].value ; i++)
+    retro_options_insert_variable (self->options, &variable_array[i]);
 
   return TRUE;
 }
@@ -435,7 +417,7 @@ on_video_refresh (guint8 *data,
                   gsize   pitch)
 {
   RetroCore *self;
-  RetroCoreEnvironmentInternal *internal;
+  RetroPixdata pixdata;
 
   if (data == NULL)
     return;
@@ -445,11 +427,12 @@ on_video_refresh (guint8 *data,
   if (self == NULL)
     g_return_if_reached ();
 
-  internal = RETRO_CORE_ENVIRONMENT_INTERNAL (self);
+  retro_pixdata_init (&pixdata,
+                      data, self->pixel_format,
+                      pitch, width, height,
+                      self->aspect_ratio);
 
-  g_signal_emit_by_name (self, "video_output", data, pitch * height,
-                         width, height, pitch, internal->pixel_format,
-                         internal->aspect_ratio);
+  g_signal_emit_by_name (self, "video-output", &pixdata);
 }
 
 // TODO This is internal, make it private as soon as possible.
@@ -464,7 +447,6 @@ on_audio_sample (gint16 left,
                  gint16 right)
 {
   RetroCore *self;
-  RetroCoreEnvironmentInternal *internal;
   gint16 samples[] = { left, right };
 
   self = retro_core_get_cb_data ();
@@ -472,12 +454,10 @@ on_audio_sample (gint16 left,
   if (self == NULL)
     g_return_if_reached ();
 
-  internal = RETRO_CORE_ENVIRONMENT_INTERNAL (self);
-
-  if (internal->sample_rate <= 0.0)
+  if (self->sample_rate <= 0.0)
     return;
 
-  g_signal_emit_by_name (self, "audio_output", samples, 2, internal->sample_rate);
+  g_signal_emit_by_name (self, "audio_output", samples, 2, self->sample_rate);
 }
 
 static gsize
@@ -485,19 +465,16 @@ on_audio_sample_batch (gint16 *data,
                        int     frames)
 {
   RetroCore *self;
-  RetroCoreEnvironmentInternal *internal;
 
   self = retro_core_get_cb_data ();
 
   if (self == NULL)
     g_return_val_if_reached (0);
 
-  internal = RETRO_CORE_ENVIRONMENT_INTERNAL (self);
-
-  if (internal->sample_rate <= 0.0)
+  if (self->sample_rate <= 0.0)
     return 0;
 
-  g_signal_emit_by_name (self, "audio_output", data, frames * 2, internal->sample_rate);
+  g_signal_emit_by_name (self, "audio_output", data, frames * 2, self->sample_rate);
 
   // FIXME What should be returned?
   return 0;
@@ -507,19 +484,13 @@ static void
 on_input_poll ()
 {
   RetroCore *self;
-  RetroInput *input;
 
   self = retro_core_get_cb_data ();
 
   if (self == NULL)
     g_return_if_reached ();
 
-  input = retro_core_get_input_interface (self);
-
-  if (input == NULL)
-    return;
-
-  retro_input_poll (input);
+  retro_core_poll_controllers (self);
 }
 
 static gint16
@@ -529,32 +500,23 @@ on_input_state (guint port,
                 guint id)
 {
   RetroCore *self;
-  RetroInput *input;
 
   self = retro_core_get_cb_data ();
 
   if (self == NULL)
     g_return_val_if_reached (0);
 
-  input = retro_core_get_input_interface (self);
-
-  if (input == NULL)
-    return 0;
-
-  return retro_input_get_state (input, port, device, index, id);
+  return retro_core_get_controller_input_state (self, port, device, index, id);
 }
 
 // TODO This is internal, make it private as soon as possible.
 void
 retro_core_set_environment_interface (RetroCore *self)
 {
-  RetroCoreEnvironmentInternal *internal;
   RetroModule *module;
   RetroCallbackSetter set_environment;
 
-  internal = RETRO_CORE_ENVIRONMENT_INTERNAL (self);
-
-  module = internal->module;
+  module = self->module;
   set_environment = retro_module_get_set_environment (module);
 
   retro_core_push_cb_data (self);
@@ -566,7 +528,6 @@ retro_core_set_environment_interface (RetroCore *self)
 void
 retro_core_set_callbacks (RetroCore *self)
 {
-  RetroCoreEnvironmentInternal *internal;
   RetroModule *module;
   RetroCallbackSetter set_video_refresh;
   RetroCallbackSetter set_audio_sample;
@@ -574,9 +535,7 @@ retro_core_set_callbacks (RetroCore *self)
   RetroCallbackSetter set_input_poll;
   RetroCallbackSetter set_input_state;
 
-  internal = RETRO_CORE_ENVIRONMENT_INTERNAL (self);
-
-  module = internal->module;
+  module = self->module;
   set_video_refresh = retro_module_get_set_video_refresh (module);
   set_audio_sample = retro_module_get_set_audio_sample (module);
   set_audio_sample_batch = retro_module_get_set_audio_sample_batch (module);
@@ -597,18 +556,14 @@ void
 retro_core_set_system_av_info (RetroCore         *self,
                                RetroSystemAvInfo *system_av_info)
 {
-  RetroCoreEnvironmentInternal *internal;
-
-  internal = RETRO_CORE_ENVIRONMENT_INTERNAL (self);
-
-  if (self->_frames_per_second != system_av_info->timing.fps) {
-    self->_frames_per_second = system_av_info->timing.fps;
+  if (self->frames_per_second != system_av_info->timing.fps) {
+    self->frames_per_second = system_av_info->timing.fps;
     g_object_notify (G_OBJECT (self), "frames-per-second");
   }
   if (system_av_info->geometry.aspect_ratio > 0.f)
-    internal->aspect_ratio = system_av_info->geometry.aspect_ratio;
+    self->aspect_ratio = system_av_info->geometry.aspect_ratio;
   else
-    internal->aspect_ratio = (float) system_av_info->geometry.base_width /
+    self->aspect_ratio = (float) system_av_info->geometry.base_width /
                              (float) system_av_info->geometry.base_height;
-  internal->sample_rate = system_av_info->timing.sample_rate;
+  self->sample_rate = system_av_info->timing.sample_rate;
 }
